ScriptName mzinBatheQuest Extends Quest
{ this script handles some functions needed by other scripts }

import mzinWaterUtil

mzinInit Property Init Auto
mzinBatheMCMMenu Property Menu Auto
mzinInterfaceSexlab Property SexlabInt Auto
mzinOverlayUtility Property Util Auto

mzinInterfaceFrostfall Property Frostfall Auto
mzinInterfacePaf Property Paf Auto
mzinInterfaceFadeTats Property FadeTats Auto

GlobalVariable Property BathingInSkyrimEnabled Auto
GlobalVariable Property WaterRestrictionEnabled Auto
GlobalVariable Property GetSoapyStyle Auto
GlobalVariable Property GetSoapyStyleFollowers Auto

GlobalVariable Property BatheKeyCode Auto
GlobalVariable Property ShowerKeyCode Auto
GlobalVariable Property CheckStatusKeyCode Auto

GlobalVariable Property DirtinessPercentage Auto

FormList Property WashPropList Auto
FormList Property SoapBonusSpellList Auto
FormList Property BatheAttemptSpellList Auto

FormList Property DirtinessSpellList Auto
FormList Property DirtinessThresholdList Auto

FormList Property WaterList Auto
FormList Property WaterfallList Auto

FormList Property SoapBonusMessageList Auto

Keyword Property SoapKeyword Auto

Spell Property DefaultBatheAttemptSpell Auto

Spell Property PlayBatheAnimationWithSoap Auto
Spell Property PlayBatheAnimationWithoutSoap Auto
Spell Property PlayShowerAnimationWithSoap Auto
Spell Property PlayShowerAnimationWithoutSoap Auto
Spell Property SoapyAppearanceSpell Auto
Spell Property SoapyAppearanceAnimatedSpell Auto

FormList Property GetDirtyOverTimeSpellList Auto

Message Property BathingWithSoapMessage Auto
Message Property BathingWithoutSoapMessage Auto
Message Property ShoweringWithSoapMessage Auto
Message Property ShoweringWithoutSoapMessage Auto

Message Property ShoweringNeedsWaterMessage Auto
Message Property DirtinessStatusMessage Auto

Actor Property PlayerRef Auto

Quest Property mzinGawkers Auto

Bool DawnguardSupportAdded
Bool DragonbornSupportAdded
Bool FalskaarSupportAdded
Bool RealisticWaterTwoSupportAdded

Event OnInit()
	RegisterHotKeys()
	RegForEvents()
EndEvent

Function RegForEvents()
	RegisterForModEvent("BiS_WashActor", "OnBiS_WashActor")
EndFunction

;/
Function SendWashPlayerEvent()
	Int WashActor = ModEvent.Create("BiS_WashActor")
	if (WashActor)
		ModEvent.PushForm(WashActor, (PlayerRef as Form))
		ModEvent.PushBool(WashActor, false)
		ModEvent.PushBool(WashActor, true)
		ModEvent.PushBool(WashActor, true)
		ModEvent.Send(WashActor)
    endIf
EndFunction
/;

Event OnBiS_WashActor(Form akActor, Bool Animate = false, Bool FullClean = false, Bool DoSoap = false)
	;Debug.Messagebox("Receive event")
	If akActor as Actor
		BatheActor(akActor as Actor, None, Animate, FullClean, DoSoap)
	Else
		Debug.Trace("Mzin: OnBiS_WashActor(): Received invalid actor: " + akActor)
	EndIf
EndEvent

Event OnReset()
	UpdateDangerousWater()
EndEvent

Event OnKeyDown(Int KeyCode)
	If Utility.IsInMenuMode() || BathingInSkyrimEnabled.GetValue() As Bool == False
		Return
	EndIf
	
	If KeyCode == CheckStatusKeyCode.GetValueInt()
		;SendWashPlayerEvent()
		DirtinessStatusMessage.Show(DirtinessPercentage.GetValue() * 100)
	ElseIf KeyCode == BatheKeyCode.GetValueInt()
		TryBatheActor(Game.GetPlayer(), None)
	ElseIf KeyCode == ShowerKeyCode.GetValueInt()
		If TryShowerActor(Game.GetPlayer(), None) == False
			ShoweringNeedsWaterMessage.Show()
		EndIf
	Endif
EndEvent

Function RegisterHotKeys()
	If BatheKeyCode.GetValueInt() != 0
		RegisterForKey(BatheKeyCode.GetValueInt())
	EndIf
	If ShowerKeyCode.GetValueInt() != 0
		RegisterForKey(ShowerKeyCode.GetValueInt())
	EndIf
	If CheckStatusKeyCode.GetValueInt() != 0
		RegisterForKey(CheckStatusKeyCode.GetValueInt())
	EndIf
EndFunction

Function UnRegisterHotKeys()
	UnregisterForKey(BatheKeyCode.GetValueInt())
	UnregisterForKey(ShowerKeyCode.GetValueInt())
	UnregisterForKey(CheckStatusKeyCode.GetValueInt())
EndFunction

Function TryBatheActor(Actor DirtyActor, MiscObject WashProp)
	;Debug.Messagebox("TryBatheActor: DirtyActor" + DirtyActor + "\nWashProp: " +WashProp )
	UnRegisterHotKeys()
	If !IsDeviceBlocked(DirtyActor)
		If IsPermitted(DirtyActor)
			If !IsTooShy(DirtyActor)
				Spell BatheAttemptSpell = DefaultBatheAttemptSpell
				If WashProp == None
					WashProp = TryFindWashProp(DirtyActor)
				EndIf
				If WashProp
					BatheAttemptSpell = BatheAttemptSpellList.GetAt(GetWashPropIndex(WashProp)) As Spell
				EndIf
				DirtyActor.AddSpell(BatheAttemptSpell, False)
				Utility.Wait(1.0)
				DirtyActor.RemoveSpell(BatheAttemptSpell)
			EndIf
		EndIf
	EndIf
	RegisterHotKeys()
EndFunction

Bool Function TryShowerActor(Actor DirtyActor, MiscObject WashProp)
	UnRegisterHotKeys()
	Bool Result = False
	If !IsDeviceBlocked(DirtyActor)
		If IsPermitted(DirtyActor)
			If !IsTooShy(DirtyActor)
				If WashProp == None
					WashProp = TryFindWashProp(DirtyActor)
				EndIf

				If WaterRestrictionEnabled.GetValue() As Bool == False || IsUnderWaterfall(DirtyActor)
					ShowerActor(DirtyActor, WashProp)
					Result = True
				EndIf
			EndIf
		EndIf
	EndIf
	RegisterHotKeys()
	Return Result
EndFunction

Function BatheActor(Actor DirtyActor, MiscObject WashProp, Bool Animate = true, Bool FullClean = false, Bool DoSoap = false)
	ForbidSex(DirtyActor, Forbid = true)
	Bool DirtyActorIsPlayer = (DirtyActor == PlayerRef)
	Bool UsedSoap = false
	DirtyActor.ClearExtraArrows()
	If DirtyActorIsPlayer
		Frostfall.MakeWet(1000.0)
		Paf.ClearPafDirt(DirtyActor)
	EndIf

	;SendCleanDirtEvent(DirtyActor)
	;Utility.Wait(0.1)

	If Animate
		If WashProp && WashProp.HasKeyWord(SoapKeyword)
			UsedSoap = true
			;ApplySoapBonus(DirtyActor, WashProp)
			DirtyActor.RemoveItem(WashProp, 1, True, None)
			DirtyActor.AddSpell(PlayBatheAnimationWithSoap, False)
			If DirtyActorIsPlayer
				BathingWithSoapMessage.Show()
			EndIf
		
		Else
			DirtyActor.AddSpell(PlayBatheAnimationWithoutSoap, False)	
			If DirtyActorIsPlayer
				BathingWithoutSoapMessage.Show()
			EndIf
		EndIf
	EndIf
	
	If FullClean
		UsedSoap = true
	EndIf
	If !Animate && DoSoap
		GetSoapy(DirtyActor)
	EndIf
	
	SendCleanDirtEvent(DirtyActor, UsedSoap)
	
	Utility.Wait(Menu.TimeToClean + 3.0 + 0.5) ; 3.0 = Wait time before dirt begins fading. See OnBiS_CleanActorDirt in mzinGetDirtyOverTime. + 0.5 = margin
	
	RemoveSpells(DirtyActor, SoapBonusSpellList)
	RemoveSpells(DirtyActor, DirtinessSpellList)
	RemoveSpells(DirtyActor, BatheAttemptSpellList)
	RemoveSpells(DirtyActor, GetDirtyOverTimeSpellList)
	;StorageUtil.UnSetStringValue(DirtyActor, "mzin_DirtTexturePrefix")
	StorageUtil.SetFloatValue(DirtyActor, "BiS_LastUpdate", Utility.GetCurrentGameTime())
	Utility.Wait(1.0)
	
	If UsedSoap
		ApplySoapBonus(DirtyActor, WashProp)
		DirtyActor.AddSpell(GetDirtyOverTimeSpellList.GetAt(0) As Spell, False)
		If DirtyActor != PlayerRef
			StorageUtil.SetFloatValue(DirtyActor, "BiS_Dirtiness", 0.0)
		EndIf
	Else
		DirtyActor.AddSpell(GetDirtyOverTimeSpellList.GetAt(1) As Spell, False)
		If DirtyActor != PlayerRef
			StorageUtil.SetFloatValue(DirtyActor, "BiS_Dirtiness", (DirtinessThresholdList.GetAt(0) As GlobalVariable).GetValue())
		EndIf
	EndIf
	If Init.IsSexlabInstalled
		SexlabInt.SlClearCum(DirtyActor)
	EndIf
	
	;StorageUtil.UnSetStringValue(DirtyActor, "mzin_DirtTexturePrefix")
	Util.SendBatheModEvent(DirtyActor as Form)
	FadeTats.FadeTats(DirtyActor, UsedSoap)
	ForbidSex(DirtyActor, Forbid = false)
	
	If DoSoap
		GetUnsoapy(DirtyActor)
	EndIf
EndFunction

Function ShowerActor(Actor DirtyActor, MiscObject WashProp)
	ForbidSex(DirtyActor, Forbid = true)
	Bool DirtyActorIsPlayer = (DirtyActor == PlayerRef)
	Bool UsedSoap = false
	DirtyActor.ClearExtraArrows()
	If DirtyActorIsPlayer
		Frostfall.MakeWet(1000.0)
		Paf.ClearPafDirt(DirtyActor)
	EndIf

	;SendCleanDirtEvent(DirtyActor)
	;Utility.Wait(0.1)

	If WashProp && WashProp.HasKeyWord(SoapKeyword)
		UsedSoap = true
		;ApplySoapBonus(DirtyActor, WashProp)
		DirtyActor.RemoveItem(WashProp, 1, True, None)
		;DirtyActor.AddSpell(GetDirtyOverTimeSpellList.GetAt(0) As Spell, False)
		DirtyActor.AddSpell(PlayShowerAnimationWithSoap, False)
		If DirtyActorIsPlayer
			ShoweringWithSoapMessage.Show()
		EndIf
	Else
		;DirtyActor.AddSpell(GetDirtyOverTimeSpellList.GetAt(1) As Spell, False)
		DirtyActor.AddSpell(PlayShowerAnimationWithoutSoap, False)
		If DirtyActorIsPlayer
			ShoweringWithoutSoapMessage.Show()
		EndIf
	EndIf
	
	SendCleanDirtEvent(DirtyActor, UsedSoap)
	
	Utility.Wait(Menu.TimeToClean + 3.0 + 0.5)
	
	RemoveSpells(DirtyActor, SoapBonusSpellList)
	RemoveSpells(DirtyActor, DirtinessSpellList)
	RemoveSpells(DirtyActor, BatheAttemptSpellList)
	RemoveSpells(DirtyActor, GetDirtyOverTimeSpellList)
	;StorageUtil.UnSetStringValue(DirtyActor, "mzin_DirtTexturePrefix")
	StorageUtil.SetFloatValue(DirtyActor, "BiS_LastUpdate", Utility.GetCurrentGameTime())
	Utility.Wait(1.0)
	
	If UsedSoap
		ApplySoapBonus(DirtyActor, WashProp)
		DirtyActor.AddSpell(GetDirtyOverTimeSpellList.GetAt(0) As Spell, False)
		If DirtyActor != PlayerRef
			StorageUtil.SetFloatValue(DirtyActor, "BiS_Dirtiness", 0.0)
		EndIf
	Else
		DirtyActor.AddSpell(GetDirtyOverTimeSpellList.GetAt(1) As Spell, False)
		If DirtyActor != PlayerRef
			StorageUtil.SetFloatValue(DirtyActor, "BiS_Dirtiness", (DirtinessThresholdList.GetAt(0) As GlobalVariable).GetValue())
		EndIf
	EndIf
	If Init.IsSexlabInstalled
		SexlabInt.SlClearCum(DirtyActor)
	EndIf
	
	;StorageUtil.UnSetStringValue(DirtyActor, "mzin_DirtTexturePrefix")
	Util.SendBatheModEvent(DirtyActor as Form)
	FadeTats.FadeTats(DirtyActor, UsedSoap)
	ForbidSex(DirtyActor, Forbid = false)
EndFunction

Function SetDirtPercentage(Actor BathingActor, Float NewDirtPercentage)
EndFunction

Function ApplySoapBonus(Actor DirtyActor, MiscObject WashProp)
	If WashProp
		Int Index = GetWashPropIndex(WashProp)
		Spell SoapBonusSpell = SoapBonusSpellList.GetAt(Index) As Spell
		DirtyActor.AddSpell(SoapBonusSpell, False)
		If DirtyActor == Game.GetPlayer()
			(SoapBonusMessageList.GetAt(Index) As Message).Show()
		EndIf
	EndIf
EndFunction

Function RemoveSpells(Actor DirtyActor, FormList SpellsFormList)
	Int Index = SpellsFormList.GetSize()
	While Index
		Index -= 1
		DirtyActor.RemoveSpell(SpellsFormList.GetAt(Index) As Spell)	
	EndWhile
EndFunction

MiscObject Function TryFindWashProp(Actor DirtyActor)
	Int WashPropIndex = WashPropList.GetSize()

	While WashPropIndex
		WashPropIndex -= 1
		MiscObject WashProp = WashPropList.GetAt(WashPropIndex) As MiscObject
		If DirtyActor.GetItemCount(WashProp) > 0
			Return WashProp
		EndIf		
	EndWhile
	
	Return None
EndFunction
Int Function GetWashPropIndex(MiscObject Soap)
	Int WashPropIndex = WashPropList.GetSize()

	While WashPropIndex
		WashPropIndex -= 1		
		If WashPropList.GetAt(WashPropIndex) As MiscObject == Soap
			Return WashPropIndex
		EndIf		
	EndWhile
	
	Return -1
EndFunction

Bool Function IsUnderWaterfall(Actor DirtyActor)
	If Game.FindClosestReferenceOfAnyTypeInListFromRef(WaterfallList, DirtyActor, 128.0)
		Return True
	EndIf

	Return False
EndFunction

Function UpdateDangerousWater()
	; try to support water from mods
	TryAddDawnguardSupport()
	TryAddDragonbornSupport()
	TryAddFalskaarSupport()
	TryAddRealisticWaterTwoSupport()

	; process the water list
	Int Index = WaterList.GetSize()
	Bool Dangerous = WaterRestrictionEnabled.GetValue() As Bool
	While Index
		Index -= 1
		Form WaterForm = WaterList.GetAt(Index)
		If WaterForm != None
			mzinWaterUtil.SetDangerousWater(WaterForm, Dangerous)
		EndIf
	EndWhile
EndFunction

Function AddWaterToList(Form WaterForm)
	If WaterForm != None && WaterList.HasForm(WaterForm) == False
		WaterList.AddForm(WaterForm)
	EndIf
EndFunction

; mod support
Function TryAddDawnguardSupport()
	If DawnguardSupportAdded == False && Game.GetModByName("Dawnguard.esm") != 255
		AddWaterToList(Game.GetFormFromFile(0x001C18, "Dawnguard.esm"))
		AddWaterToList(Game.GetFormFromFile(0x002932, "Dawnguard.esm"))
		AddWaterToList(Game.GetFormFromFile(0x006AEB, "Dawnguard.esm"))
		AddWaterToList(Game.GetFormFromFile(0x006AFD, "Dawnguard.esm"))
		AddWaterToList(Game.GetFormFromFile(0x00C7C2, "Dawnguard.esm"))
		AddWaterToList(Game.GetFormFromFile(0x00CE71, "Dawnguard.esm"))
		AddWaterToList(Game.GetFormFromFile(0x00CE72, "Dawnguard.esm"))
		AddWaterToList(Game.GetFormFromFile(0x019C98, "Dawnguard.esm"))
		Debug.Notification("Bathing in Skyrim - Dawnguard Support Added")
		DawnguardSupportAdded = True
	EndIf
EndFunction

Function TryAddDragonbornSupport()
	If DragonbornSupportAdded == False && Game.GetModByName("Dragonborn.esm") != 255
		AddWaterToList(Game.GetFormFromFile(0x0173B6, "Dragonborn.esm"))
		AddWaterToList(Game.GetFormFromFile(0x01DFF1, "Dragonborn.esm"))
		AddWaterToList(Game.GetFormFromFile(0x028644, "Dragonborn.esm"))
		AddWaterToList(Game.GetFormFromFile(0x02ADEC, "Dragonborn.esm"))
		AddWaterToList(Game.GetFormFromFile(0x03731A, "Dragonborn.esm"))
		AddWaterToList(Game.GetFormFromFile(0x03805D, "Dragonborn.esm"))
		Debug.Notification("Bathing in Skyrim - Dragonborn Support Added")
		DragonbornSupportAdded = True
	EndIf
EndFunction

Function TryAddFalskaarSupport()
	If FalskaarSupportAdded == False && Game.GetModByName("Falskaar.esm") != 255
		AddWaterToList(Game.GetFormFromFile(0x052A0F, "Falskaar.esm"))
		AddWaterToList(Game.GetFormFromFile(0x0B5AB4, "Falskaar.esm"))
		AddWaterToList(Game.GetFormFromFile(0x0B5AB5, "Falskaar.esm"))
		AddWaterToList(Game.GetFormFromFile(0x0B5AB6, "Falskaar.esm"))
		AddWaterToList(Game.GetFormFromFile(0x0B5AB7, "Falskaar.esm"))
		AddWaterToList(Game.GetFormFromFile(0x0B7DA2, "Falskaar.esm"))
		AddWaterToList(Game.GetFormFromFile(0x11960F, "Falskaar.esm"))
		AddWaterToList(Game.GetFormFromFile(0x17B736, "Falskaar.esm"))
		Debug.Notification("Bathing in Skyrim - Falskaar Support Added")
		FalskaarSupportAdded = True
	EndIf
EndFunction

Function TryAddRealisticWaterTwoSupport()
	If RealisticWaterTwoSupportAdded == False && Game.GetModByName("RealisticWaterTwo.esp") != 255
		AddWaterToList(Game.GetFormFromFile(0x002DAE, "RealisticWaterTwo.esp"))
		AddWaterToList(Game.GetFormFromFile(0x0048D3, "RealisticWaterTwo.esp"))
		AddWaterToList(Game.GetFormFromFile(0x015BE8, "RealisticWaterTwo.esp"))
		AddWaterToList(Game.GetFormFromFile(0x02088C, "RealisticWaterTwo.esp"))
		Debug.Notification("Bathing in Skyrim - Realistic Water Two Support Added")
		RealisticWaterTwoSupportAdded = True
	EndIf
EndFunction

Bool Function IsDeviceBlocked(Actor akTarget)
	If Init.IsDdsInstalled
		If akTarget.WornHasKeyword(Init.zad_DeviousHeavyBondage)
			Debug.Notification("You can't wash yourself with your hands tied")
			Return True
		ElseIf akTarget.WornHasKeyword(Init.zad_DeviousSuit)
			Debug.Notification("You can't wash yourself while wearing this suit")
			Return True
		Else
			Return False
		EndIf
	Else
		Return False
	EndIf
EndFunction

Function SendCleanDirtEvent(Form akTarget, Bool UsedSoap)
	int BiS_CleanActorDirtEvent = ModEvent.Create("BiS_CleanActorDirt")
    If (BiS_CleanActorDirtEvent)
		ModEvent.PushForm(BiS_CleanActorDirtEvent, akTarget)
		ModEvent.PushFloat(BiS_CleanActorDirtEvent, Menu.TimeToClean)
		ModEvent.PushFloat(BiS_CleanActorDirtEvent, Menu.TimeToCleanInterval)
		ModEvent.PushBool(BiS_CleanActorDirtEvent, UsedSoap)
        ModEvent.Send(BiS_CleanActorDirtEvent)
    EndIf
EndFunction

Bool Function IsPermitted(Actor akTarget)
	Int Index = StorageUtil.FormListFind(none, "BiS_ForbiddenActors", akTarget)
	If Index != -1
		Int ForbiddenCount = StorageUtil.StringListCount(akTarget, "BiS_ForbiddenString") - 1
		String ForbiddenString = StorageUtil.StringListGet(akTarget, "BiS_ForbiddenString", ForbiddenCount)
		If ForbiddenString != ""
			Debug.Notification(ForbiddenString)
		Else
			Debug.Trace("Mzin: IsPermitted: Blank string retrieved for index " + ForbiddenCount + " on actor " + akTarget)
		EndIf
		
		; Send forbidden bathe attempt modevent
		Int ForbiddenBatheAttempt = ModEvent.Create("BiS_ForbiddenBatheAttempt")
		If (ForbiddenBatheAttempt)
			ModEvent.PushForm(ForbiddenBatheAttempt, akTarget)
			ModEvent.Send(ForbiddenBatheAttempt)
		EndIf
		;StartAnimationSequence("mzinBatheA1_S1_Cloth", "mzinBatheA1_S2_Cloth_DONE")
		Debug.SendAnimationEvent(akTarget, "IdleWarmHandsCrouched")
		Utility.Wait(2.0)
		Debug.SendAnimationEvent(akTarget, "IdleStop")
		Return False
	Else
		Return True
	EndIf
EndFunction

Bool Function IsTooShy(Actor akTarget)
	If Menu.Shyness
		If Game.GetModByName("SexLabAroused.esm") != 255
			Faction ExhibitionistFact = Game.GetFormFromFile(0x0713DA, "SexLabAroused.esm") as Faction
			If ExhibitionistFact != None
				If akTarget.GetFactionRank(ExhibitionistFact) >= 0
					Return False
				EndIf
			EndIf
		EndIf
		
		mzinGawkers.Stop()
		mzinGawkers.Start()
		Utility.Wait(0.1)

		Actor Gawker
		Int i = 0 
		While i < mzinGawkers.GetNumAliases()
			Gawker = (mzinGawkers.GetNthAlias(i) as ReferenceAlias).GetReference() as Actor
			If Gawker != None && Gawker != akTarget
				If !IsGawkerSlave(Gawker)
					If Gawker.HasLOS(PlayerRef)
						DoShyMessage(akTarget, Gawker)
						Return True
					EndIf
					If !akTarget.IsInInterior()
						If akTarget.GetDistance(Gawker) < Menu.ShyDistance
							DoShyMessage(akTarget, Gawker)
							Return True
						EndIf
					EndIf
				EndIf
			EndIf
			i += 1
		EndWhile
		Return False
	
	Else
		Return False
	EndIf
EndFunction

Function DoShyMessage(Actor akTarget, Actor Gawker)
	If akTarget == PlayerRef
		Debug.Notification("No way am I bathing in front of " + Gawker.GetBaseObject().GetName())
	Else
		Debug.Notification(akTarget.GetBaseObject().GetName() + ": You're joking right? I'm not bathing in front of " + Gawker.GetBaseObject().GetName())
	EndIf
EndFunction

Bool Function IsGawkerSlave(Actor Gawker)
	If Init.IsZazInstalled
		If Gawker.IsInFaction(Init.ZazSlaveFaction)
			Return True
		Else
			Return False
		EndIf
	Else
		Return False
	EndIf
EndFunction

Function ForbidSex(Actor akTarget, Bool Forbid)
	If Init.IsSexlabInstalled && akTarget
		Faction SexLabForbiddenActors  = Game.GetFormFromFile(0x049068, "SexLab.esm") as Faction
		If Forbid
			akTarget.AddToFaction(SexLabForbiddenActors)
		Else
			akTarget.RemoveFromFaction(SexLabForbiddenActors)
		EndIf
	EndIf
EndFunction

Function GetSoapy(Actor akActor)
	If akActor == PlayerRef
		If GetSoapyStyle.GetValue() == 1
			akActor.AddSpell(SoapyAppearanceSpell, False)
		ElseIf GetSoapyStyle.GetValue() == 2
			akActor.AddSpell(SoapyAppearanceAnimatedSpell, False)
		EndIf
	Else
		If GetSoapyStyleFollowers.GetValue() == 1
			akActor.AddSpell(SoapyAppearanceSpell, False)
		ElseIf GetSoapyStyleFollowers.GetValue() == 2
			akActor.AddSpell(SoapyAppearanceAnimatedSpell, False)
		EndIf
	EndIf
EndFunction

Function GetUnsoapy(Actor akActor)
	If akActor.HasSpell(SoapyAppearanceSpell)
		akActor.RemoveSpell(SoapyAppearanceSpell)
	ElseIf akActor.HasSpell(SoapyAppearanceAnimatedSpell)
		akActor.RemoveSpell(SoapyAppearanceAnimatedSpell)
	EndIf
EndFunction
